#include "./BwTree/src/bwtree.h"
#include "../_common/benchmark.hpp"

namespace wangziqi2013 {
namespace bwtree {
bool print_flag = false;
thread_local int BwTreeBase::gc_id = -1;
std::atomic<size_t> BwTreeBase::total_thread_num{0UL};
} // namespace bwtree
} // namespace wangziqi2013

class KeyComparator {
public:
  inline bool operator()(const long int k1, const long int k2) const {
    return k1 < k2;
  }

  KeyComparator() = delete;
};

class KeyEqualityChecker {
public:
  inline bool operator()(const long int k1, const long int k2) const {
    return k1 == k2;
  }

  KeyEqualityChecker() = delete;
};

int main() {
  using KeyType = uint32_t;
  using ValueType = uint32_t;

  Benchmark::execute<Benchmark::Device::Host, KeyType>(
      "bwtree", [](auto &timer, std::vector<KeyType> input) {
        using TreeType =
            wangziqi2013::bwtree::BwTree<KeyType, ValueType, KeyComparator,
                                         KeyEqualityChecker>;

        TreeType *t = new TreeType{true, KeyComparator{}, KeyEqualityChecker{}};
        t->UpdateThreadLocal(1);
        t->AssignGCID(0);

        timer.start();
        for (auto i : input) {
          t->Insert(i, i);
        }
        timer.stop("insert");

        size_t size = input.size();
        std::vector<std::unordered_set<ValueType>> results(size);

        timer.start();
        for (auto i : input) {
          results.emplace_back(t->GetValue(i));
        }
        timer.stop("query");
      });

  return 0;
}
