#include "../_common/benchmark.hpp"

#include <limits>

#include <thrust/device_vector.h>
#include <thrust/host_vector.h>
#include <thrust/sequence.h>
#include <thrust/transform.h>

#include <cuco/static_map.cuh>

#include <stdio.h>
#include <stdlib.h>

#include <algorithm>
#include <random>
#include <vector>

int main(int argc, char *argv[]) {
  using KeyType = uint32_t;
  using ValueType = uint32_t;

  Benchmark::execute<Benchmark::Device::Cuda,
                     KeyType>("cuco", [](auto &timer,
                                         std::vector<KeyType> input) {
    size_t size = input.size();
    cuco::static_map<KeyType, ValueType> map{size * 2, UINT32_MAX, UINT32_MAX};

    thrust::host_vector<thrust::pair<KeyType, ValueType>> hostPairs(size);
    for (size_t i = 0; i < size; ++i) {
      hostPairs[i] = thrust::make_pair(input[i], input[i]);
    }
    thrust::device_vector<thrust::pair<KeyType, ValueType>> pairs(hostPairs);

    timer.start();
    map.insert(pairs.begin(), pairs.end());
    timer.stop("insert");

    thrust::device_vector<KeyType> keys_to_find(size);
    thrust::sequence(keys_to_find.begin(), keys_to_find.end(), 0);
    thrust::device_vector<ValueType> found_values(size);

    timer.start();
    map.find(keys_to_find.begin(), keys_to_find.end(), found_values.begin());
    timer.stop("query");
  });

  return 0;
}
