#include <stdio.h>
#include <stdlib.h>

#include <algorithm>
#include <random>
#include <vector>

#include "../_common/benchmark.hpp"
#include "./GpuBTree/src/GpuBTree.h"

int main(int argc, char *argv[]) {
  using KeyType = uint32_t;
  using ValueType = uint32_t;

  Benchmark::execute<Benchmark::Device::Cuda, KeyType>(
      "owg", [](auto &timer, std::vector<KeyType> input) {
        GpuBTree::GpuBTreeMap<KeyType, ValueType, uint32_t> btree;

        uint32_t numKeys = input.size();

        KeyType *gpuKeys;
        CHECK_ERROR(memoryUtil::deviceAlloc(gpuKeys, numKeys));
        CHECK_ERROR(memoryUtil::cpyToDevice(input.data(), gpuKeys, numKeys));

        ValueType *gpuValues;
        CHECK_ERROR(memoryUtil::deviceAlloc(gpuValues, numKeys));
        CHECK_ERROR(memoryUtil::cpyToDevice(input.data(), gpuValues, numKeys));

        timer.start();
        btree.insertKeys(gpuKeys, gpuValues, numKeys, SourceT::DEVICE);
        timer.stop("insert");

        cudaFree(gpuKeys);
        cudaFree(gpuValues);

        KeyType *gpuQueries;
        CHECK_ERROR(memoryUtil::deviceAlloc(gpuQueries, numKeys));
        CHECK_ERROR(memoryUtil::cpyToDevice(input.data(), gpuQueries, numKeys));

        ValueType *gpuResults;
        CHECK_ERROR(memoryUtil::deviceAlloc(gpuResults, numKeys));

        timer.start();
        btree.searchKeys(gpuQueries, gpuResults, numKeys, SourceT::DEVICE);
        timer.stop("query");

        cudaFree(gpuQueries);
        cudaFree(gpuResults);
        btree.free();
      });

  return 0;
}