#include "../_common/benchmark.hpp"

#include "../../implementation/src/BLinkTree/Default.hpp"
#include "../../implementation/src/BTreeContainer/Default.hpp"

#include <TNL/Assert.h>
#include <TNL/Containers/Array.h>
#include <TNL/Containers/ArrayView.h>
#include <TNL/Devices/Cuda.h>
#include <TNL/Exceptions/NotImplementedError.h>
#include <TNL/Pointers/DevicePointer.h>

using namespace TNL;
using namespace TNL::Containers;

int main(int argc, char **argv) {
  using KeyType = uint32_t;
  using ValueType = uint32_t;
  const size_t Order = 15;

  Benchmark::execute<Benchmark::Device::Cuda, KeyType>(
      "tnl::cuda", [](auto &timer, std::vector<KeyType> input) {
        using KeyArray = Containers::Array<ValueType, TNL::Devices::Cuda>;
        using ValueArray = Containers::Array<ValueType, TNL::Devices::Cuda>;
        using MaskArray = Containers::Array<bool, TNL::Devices::Cuda>;

        using Tree = BTreeContainer<KeyType, ValueType, Order,
                                    TNL::Devices::Cuda, BLinkTree>;

        Tree container(input.size());

        KeyArray keys(input);
        ValueArray values(input);

        container.init();

        timer.start();
        container.insert(keys, values);
        timer.stop("insert");

        ValueArray results(input.size());
        MaskArray mask(input.size());

        timer.start();
        container.find(keys, results, mask);
        timer.stop("query");
      });

  return 0;
}