/** @jsxImportSource @emotion/react */
import { useEffect, useState } from "react"
import { Playback } from "../core/node"
import { css } from "@emotion/react"
import { PlaybackLine } from "./PlaybackLine"
import { List } from "react-virtualized"

export function PlaybackHistory(props: {
  playback: Playback
  index: number
  onIndexClick: (idx: number) => void
}) {
  const [height, setHeight] = useState(0)
  const [highlight, setHighlight] = useState<Record<number, boolean>>({})

  useEffect(() => {
    const listener = () => {
      setHeight(window.innerHeight)
    }

    listener()
    window.addEventListener("resize", listener)
    return () => window.removeEventListener("resize", listener)
  }, [])

  return (
    <div
      css={css`
        position: fixed;
        top: 0;
        left: 0;
        right: 0;
        bottom: 0;
        pointer-events: none;
        overflow: hidden;

        z-index: 1;
      `}
    >
      <div
        css={css`
          position: absolute;
          right: 0;
          bottom: 0;
          border: 1px solid rgba(0, 0, 0, 0.3);
          border-radius: 8px;
          transform: translateY(90%);
          transition: all 0.3s;
          background: rgba(255, 255, 255, 0.8);
          overflow: hidden;

          pointer-events: all;

          &:hover {
            transform: translateY(0%);
          }
        `}
        onKeyDown={(e) => e.preventDefault()}
      >
        <List
          height={height}
          rowCount={props.playback.instructions.length}
          rowHeight={16}
          width={800}
          rowRenderer={({ key, index, style }) => {
            return (
              <PlaybackLine
                key={key}
                style={style}
                content={props.playback.instructions[index][0]}
                isActive={index === props.index}
                line={index + 1}
                onLineClick={() => props.onIndexClick(index)}
                isHighlight={highlight[index]}
                onHighlight={() => {
                  setHighlight((data) => ({ ...data, [index]: !data[index] }))
                }}
              />
            )
          }}
        />
      </div>
    </div>
  )
}
