/** @jsxImportSource @emotion/react */
import { CSSProperties, useEffect, useRef } from "react"

export function PlaybackLine(props: {
  content: string
  line: number
  style: CSSProperties
  isActive?: boolean
  isHighlight?: boolean
  onHighlight?: () => void
  onLineClick?: () => void
}) {
  const ref = useRef<HTMLDivElement>(null)

  useEffect(() => {
    if (!props.isActive) return
    const highlight = props.onHighlight
    const listener = (e: KeyboardEvent) => {
      if (e.key === " ") {
        highlight?.()
      }
    }

    window.addEventListener("keydown", listener)
    return () => window.removeEventListener("keydown", listener)
  }, [props.isActive, props.onHighlight])

  let style: CSSProperties = {
    ...props.style,
    whiteSpace: "nowrap",
    fontSize: 12,
  }

  style.color = props.content.includes("!!!") ? "#555" : "#ccc"

  if (props.isActive) {
    style.fontWeight = 700
    style.background = "#eee"
    style.color = "#000"
  }

  if (props.isHighlight) {
    style.background = "#aa0000"
    style.color = "#fff"
  }

  return (
    <div ref={ref} style={style}>
      <span
        style={{
          borderRight: "1px solid black",
          padding: "0 5px",
          marginRight: 5,
          textAlign: "right",
          display: "inline-block",
          width: 30,
          cursor: "pointer",
          background: "rgba(0,0,0,0.05)",
        }}
        onClick={props.onLineClick}
      >
        {props.line}
      </span>
      <span onClick={props.onHighlight}>{props.content}</span>
    </div>
  )
}
