#!/usr/bin/node
const fs = require("fs")
const path = require("path")

const readFilePath = path.resolve(__dirname, "../compile_commands.json")

let config = JSON.parse(
  fs.readFileSync(readFilePath, {
    encoding: "utf-8",
  })
)

config = config.map((target) => {
  if (target.file.endsWith(".cu")) {
    const segments = target.command.split(" ").map((i) => i.trim())

    if (segments[0].includes("nvcc")) {
      return {
        ...target,
        command: segments
          .filter((segment) => {
            if (
              segment === "-Xcudafe" ||
              segment === "--expt-relaxed-constexpr" ||
              segment === "--expt-extended-lambda" ||
              segment === "--diag_suppress=esa_on_defaulted_function_ignored" ||
              segment === "-gencode" ||
              segment === "-forward-unknown-to-host-compiler"
            ) {
              return false
            }

            return true
          })
          .map((segment) => {
            if (segment.startsWith("-isystem=")) {
              return ["-I", segment.split("-isystem=").pop()].join("")
            }

            if (segment.startsWith("--generate-code=")) {
              const value = segment.split("--generate-code=").pop()
              const res = value.match(/sm_([0-9]{2})/)
              if (!value.includes("sm_") || !res) {
                return null
              }

              return `--cuda-gpu-arch=${res[0]}`
            }
            return segment
          })
          .filter(Boolean)
          .join(" "),
      }
    }
  }

  return target
})

fs.writeFileSync(readFilePath, JSON.stringify(config, null, 2), {
  encoding: "utf-8",
})
