#!/usr/bin/node
const fs = require("fs");
const path = require("path");

const filePath = path.resolve(__dirname, "../compile_commands.json");
const config = JSON.parse(
  fs.readFileSync(filePath, {
    encoding: "utf-8",
  })
);

const tasks = {
  version: "2.0.0",
  tasks: [],
};

const launch = {
  version: "0.2.0",
  configurations: [],
};

for (const { file, command } of config) {
  const variant = path
    .basename(path.dirname(path.dirname(file)))
    .toLocaleLowerCase();
  const filetarget = path.basename(file).replace(path.extname(file), "");

  const target = `${filetarget}`;
  const device = command.includes("/nvcc") ? "cuda" : "host";

  tasks.tasks.push({
    label: `build:${target}`,
    type: "shell",
    command: `make ${target}`,
    problemMatcher: [],
    group: {
      kind: "build",
      isDefault: true,
    },
  });

  launch.configurations.push({
    name: device === "cuda" ? `(cuda-gdb) ${target}` : `(gdb) ${target}`,
    miDebuggerPath:
      device === "cuda" ? "/usr/local/cuda/bin/cuda-gdb" : "/usr/bin/gdb",
    program: `$\{workspaceFolder\}/bin/${target}`,
    preLaunchTask: `build:${target}`,
    type: "cppdbg",
    request: "launch",
    args: [],
    stopAtEntry: false,
    cwd: "${workspaceFolder}",
    MIMode: "gdb",
    setupCommands: [
      {
        description: "Enable pretty-printing for gdb",
        text: "-enable-pretty-printing",
        ignoreFailures: true,
      },
    ],
  });
}

try {
  fs.mkdirSync(path.resolve(__dirname, "../.vscode"));
} catch (err) {}

fs.writeFileSync(
  path.resolve(__dirname, "../.vscode/launch.json"),
  JSON.stringify(launch, null, 2),
  { encoding: "utf-8" }
);

fs.writeFileSync(
  path.resolve(__dirname, "../.vscode/tasks.json"),
  JSON.stringify(tasks, null, 2),
  { encoding: "utf-8" }
);
