#pragma once
#include <TNL/Assert.h>
#include <TNL/Cuda/CudaCallable.h>
#include <TNL/TypeTraits.h>
#include <iostream>
#include <sstream>
#include <stdint.h>
#include <string>
#include <vector>

#include "../../Utils/Algorithms.hpp"
#include "../BLinkNode.hpp"

template <typename KeyType, typename ValueType, size_t Order, typename Device>
struct BLinkOperations;

template <typename KeyType, typename ValueType, size_t Order>
struct BLinkOperations<KeyType, ValueType, Order, TNL::Devices::Host> {
  using Node = BLinkNode<KeyType, ValueType, Order>;

  static void init(Node *volatile node, bool leaf, Node *volatile sibling, uint8_t highKeyFlag) {
    node->mLeaf = leaf;
    node->mSibling = sibling;
    node->mHighKeyFlag = highKeyFlag;
    node->mSize = 0;
  }

  static void init(Node *volatile node, Node *volatile cursor) {
    init(node, cursor->mLeaf, cursor->mSibling, cursor->mHighKeyFlag);
    node->mHighKey = cursor->mHighKey;
  }

  static void insertKey(Node *volatile node, size_t index, KeyType key, ValueType value) {
    TNL_ASSERT_TRUE(node->mLeaf, "Inserting value is only supported in leaves");
    insert(node->mKeys, node->mSize, index, key);
    insert(node->mValues, node->mSize, index, value);
    node->mSize += 1;
  }

  static void insertKey(Node *volatile node, size_t index, KeyType key) {
    TNL_ASSERT_FALSE(node->mLeaf, "Inserting with no value in an internal node");
    insert(node->mKeys, node->mSize, index, key);
    node->mSize += 1;
  }

  static void appendKey(Node *volatile node, KeyType key, ValueType value) {
    TNL_ASSERT_TRUE(node->mLeaf, "Inserting value is only supported in leaves");
    node->mKeys[node->mSize] = key;
    node->mValues[node->mSize] = value;
    node->mSize += 1;
  }

  static void appendKey(Node *volatile node, KeyType key) {
    TNL_ASSERT_FALSE(node->mLeaf, "Inserting with no value in an internal node");
    node->mKeys[node->mSize] = key;
    node->mSize += 1;
  }

  static void removeKey(Node *volatile node, size_t index) {
    if (node->mSize > 0) {
      remove(node->mKeys, node->mSize, index);
      remove(node->mValues, node->mSize, index);
      node->mSize -= 1;
    }
  }

  static void insertChild(Node *volatile node, size_t index, Node *child, size_t childSize) {
    for (int i = childSize - 1; i >= index && i >= 0; --i) {
      node->mChildren[i + 1] = node->mChildren[i];
    }

    node->mChildren[index] = child;
  }

  static void appendChild(Node *volatile node, Node *volatile child, size_t childSize) {
    node->mChildren[childSize] = child;
  }

  static void removeChild(Node *volatile node, size_t index, size_t childSize) {
    remove(node->mChildren, childSize, index);
  }
};