#pragma once
#include "TNL/Cuda/CudaCallable.h"
#include <TNL/Devices/Cuda.h>
#include <TNL/Devices/Host.h>
#include <TNL/Exceptions/CudaSupportMissing.h>

#include "../Constants.hpp"
#include "../Utils/Debugger.hpp"

using namespace TNL;

template <typename Object, typename Device> struct BNodeLatch;

template <template <typename KeyType, typename ValueType, size_t Order> class BNode,
          typename KeyType, typename ValueType, size_t Order>
struct BNodeLatch<BNode<KeyType, ValueType, Order>, Devices::Cuda> {
  using BTreeNode = BNode<KeyType, ValueType, Order>;

#ifdef HAVE_CUDA
  __device__ void acquire(BTreeNode *volatile node) {
    bool locked = true;

    while (locked) {
      if (atomicCAS(&node->mWriteLock, false, true) == false) {
        DBG_LATCH_ACQUIRE(node);
        break;
      }
    }
  }

  __device__ bool attempt(BTreeNode *volatile node) {
    if (atomicCAS(&node->mWriteLock, false, true) == false) {
      DBG_LATCH_ATTEMPT(node);
      return true;
    }
    return false;
  }

  __device__ void release(BTreeNode *volatile node) {
    DBG_LATCH_RELEASE(node);
    atomicAnd(&node->mWriteLock, false);
  }

#else  // HAVE_CUDA
  void acquire(BTreeNode *volatile node) { throw TNL::Exceptions::CudaSupportMissing(); }

  bool attempt(BTreeNode *volatile node) { throw TNL::Exceptions::CudaSupportMissing(); }

  void release(BTreeNode *volatile node) { throw TNL::Exceptions::CudaSupportMissing(); }
#endif // HAVE_CUDA
};