#pragma once
#include <TNL/Containers/ArrayView.h>
#include <TNL/TypeTraits.h>

template <typename T> __cuda_callable__ size_t lowerBound(const T *view, size_t size, T value) {
  size_t it = 0, first = 0, step = 0;

  while (size > 0) {
    it = first;
    step = size / 2;
    it += step;

    if (view[it] < value) {
      first = ++it;
      size -= step + 1;
    } else {
      size = step;
    }
  }

  return first;
}

template <typename T> __cuda_callable__ size_t upperBound(const T *view, size_t size, T value) {
  size_t it = 0, first = 0, step = 0;

  while (size > 0) {
    step = size / 2;
    it = first + step;

    if (!(value < view[it])) {
      first = ++it;
      size -= step + 1;
    } else {
      size = step;
    }
  }

  return first;
}

template <typename T> __cuda_callable__ T *insert(T *array, size_t size, size_t index, T value) {
  for (int i = size - 1; i >= index && i >= 0; --i) {
    array[i + 1] = array[i];
  }

  array[index] = value;
  return array;
}

template <typename T> __cuda_callable__ T *remove(T *array, size_t size, size_t index) {
  for (int i = index + 1; i < size && i > 0; ++i) {
    array[i - 1] = array[i];
  }

  return array;
}

inline size_t getContainerSize(size_t keyCount, size_t order) {
  size_t size = order - 1;
  size_t count = keyCount;
  size_t sum = 0;
  while (count > 1) {
    sum += count = ((count + size - 1) / size);
  }

  int total = 1;
  while (total < sum) {
    total <<= 1;
  }
  return total * 2;
}