#pragma once

#include "../Constants.hpp"

#ifdef DEBUGGER
#define DBG_INIT(root, order)                                                                      \
  printf("thr[%d,%d]: !!! INIT root(0x%lx, %lu)\n", blockIdx.x, threadIdx.x, ADDR(root), order)

#define DBG_LATCH_ACQUIRE(addr)                                                                    \
  printf("thr[%d,%d]: !!! LATCH acquire(0x%lx)\n", blockIdx.x, threadIdx.x, ADDR(addr))

#define DBG_LATCH_ATTEMPT(addr)                                                                    \
  printf("thr[%d,%d]: !!! LATCH attempt(0x%lx)\n", blockIdx.x, threadIdx.x, ADDR(addr))

#define DBG_LATCH_RELEASE(addr)                                                                    \
  printf("thr[%d,%d]: !!! LATCH release(0x%lx)\n", blockIdx.x, threadIdx.x, ADDR(addr))

#define DBG_INSERT_INTERNAL(addr, insertKey, childAddr, index, childIndex)                         \
  printf("thr[%d,%d]: !!! INSERT insertInternal(0x%lx, %lu, 0x%lx, %lu, %lu)\n", blockIdx.x,       \
         threadIdx.x, ADDR(addr), insertKey, ADDR(childAddr), index, childIndex)

#define DBG_INSERT_LEAF(addr, insertKey, index)                                                    \
  printf("thr[%d,%d]: !!! INSERT insertLeaf(0x%lx, %lu, %lu)\n", blockIdx.x, threadIdx.x,          \
         ADDR(addr), insertKey, index)

#define DBG_INCREASE_HEIGHT(leftAddr, rootAddr, rightAddr, topKey)                                 \
  printf("thr[%d,%d]: !!! SPLIT increaseHeight(0x%lx, 0x%lx, 0x%lx, %lu) <%lu, %lu, %lu> "         \
         "[%lu, %lu, %lu]\n",                                                                      \
         blockIdx.x, threadIdx.x, ADDR(leftAddr), ADDR(rootAddr), ADDR(rightAddr), topKey,         \
         leftAddr->mHighKey, rootAddr->mHighKey, rightAddr->mHighKey, leftAddr->size(),            \
         rootAddr->size(), rightAddr->size())

#define DBG_SPLIT(source, startIdx, leftLength, targetAddr)                                        \
  printf("thr[%d,%d]: !!! SPLIT split(0x%lx, %lu, %lu, 0x%lx) <%lu, %lu> [%lu, %lu]\n",            \
         blockIdx.x, threadIdx.x, ADDR(source), startIdx, leftLength, ADDR(targetAddr),            \
         source->size(), targetAddr->size(), source->mHighKey, targetAddr->mHighKey)

#define DBG_DELETE(addr, index)                                                                    \
  printf("thr[%d,%d]: !!! DELETE delete(0x%lx, %lu)\n", blockIdx.x, threadIdx.x, ADDR(addr), index)

#else

#define DBG_INIT(root, order)
#define DBG_LATCH_ACQUIRE(addr)
#define DBG_LATCH_ATTEMPT(addr)
#define DBG_LATCH_RELEASE(addr)
#define DBG_INSERT_INTERNAL(addr, insertKey, childAddr, index, childIndex)
#define DBG_INSERT_LEAF(addr, insertKey, index)
#define DBG_INCREASE_HEIGHT(leftAddr, rootAddr, rightAddr, topKey)
#define DBG_SPLIT(source, startIdx, leftLength, targetAddr)
#define DBG_DELETE(addr, index)

#endif