#pragma once
#include <TNL/Algorithms/MemoryOperations.h>
#include <TNL/Allocators/Default.h>
#include <TNL/Cuda/CudaCallable.h>
#include <TNL/Devices/Host.h>
#include <TNL/Exceptions/NotImplementedError.h>

using namespace TNL;

template <typename Value, typename Device = TNL::Devices::Host> class DeviceVariable {
  using VariableType = DeviceVariable<Value, Device>;
  using AllocatorType = typename Allocators::Default<Device>::template Allocator<Value>;

public:
  Value *mPtr;

  DeviceVariable() : mPtr(AllocatorType{}.allocate(1)){};

  DeviceVariable(Value value) : DeviceVariable() {
    TNL::Algorithms::MemoryOperations<Device>::setElement(mPtr, value);
  };

  __cuda_callable__ operator Value() const {
    return TNL::Algorithms::MemoryOperations<Device>::getElement(mPtr);
  };

  __cuda_callable__ Value operator*() const {
    return TNL::Algorithms::MemoryOperations<Device>::getElement(mPtr);
  };

  __cuda_callable__ Value operator->() const {
    return TNL::Algorithms::MemoryOperations<Device>::getElement(mPtr);
  };

  __cuda_callable__ DeviceVariable &operator=(Value v) {
    TNL::Algorithms::MemoryOperations<Device>::setElement(mPtr, v);
    return *this;
  }
};
