
#include "Utils/Algorithms.hpp"

#include <TNL/Containers/Array.h>
#include <TNL/Containers/ArrayView.h>
#include <gtest/gtest.h>

using namespace TNL;
using namespace TNL::Containers;

TEST(LowerBound, TNLArray) {
  Array<int> hostArray(10);
  for (int i = 0; i < 10; ++i) {
    hostArray[i] = i * 10;
  }

  auto view = hostArray.getConstView();
  for (size_t expect = 0; expect < 10; ++expect) {
    int value = view[expect];
    size_t result = lowerBound<int>(view.getArrayData(), view.getSize(), value);
    EXPECT_EQ(expect, result);
  }
}

TEST(UpperBound, TNLArray) {
  Array<int> hostArray(10);
  for (int i = 0; i < 10; ++i) {
    hostArray[i] = i * 10;
  }

  auto view = hostArray.getConstView();
  for (size_t expect = 0; expect < 10; ++expect) {
    int value = view[expect];
    size_t result = upperBound<int>(view.getArrayData(), view.getSize(), value);
    EXPECT_EQ(expect + 1, result);
  }
}

TEST(Insert, Array) {
  int test[3] = {1, 2, 0};
  insert(test, 2, 1, 5);

  EXPECT_EQ(test[0], 1);
  EXPECT_EQ(test[1], 5);
  EXPECT_EQ(test[2], 2);
}

TEST(Insert, Empty) {
  int test[3];
  insert(test, 0, 0, 5);

  EXPECT_EQ(test[0], 5);
}

TEST(Remove, Array) {
  int test[3] = {1, 5, 2};
  remove(test, 3, 1);

  EXPECT_EQ(test[0], 1);
  EXPECT_EQ(test[1], 2);
  EXPECT_EQ(test[2], 2);
}

TEST(LowerBound, ArrayRandom) {
  int test[9] = {35005211,  42999170,  84353895,  112805732, 135497281,
                 137806862, 149798315, 213975407, 269441500};
  EXPECT_EQ(lowerBound(test, 9, 35005211), 0);
}

int main(int argc, char **argv) {
  testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();
}
