#include <TNL/Containers/Array.h>
#include <TNL/Containers/StaticArray.h>
#include <TNL/Containers/Vector.h>
#include <bits/stdint-uintn.h>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <limits>
#include <list>
#include <map>
#include <sstream>
#include <string>
#include <utility>

#include "BLinkTree/Default.hpp"
#include "BPlusTree/Default.hpp"
#include "BTreeContainer/Default.hpp"
#include "TNL/Algorithms/ParallelFor.h"
#include "TNL/Cuda/CudaCallable.h"
#include <TNL/Containers/Array.h>

#include <gtest/gtest.h>

struct Point {
  int x, y, z;
  __cuda_callable__ bool operator<(const Point &rhs) const {
    return x < rhs.x || y < rhs.y || z < rhs.z;
  }

  __cuda_callable__ bool operator==(const Point &rhs) const {
    return x == rhs.x && y == rhs.y && z == rhs.z;
  }

  friend std::ostream &operator<<(std::ostream &output, const Point &item) {
    output << item.x << ", " << item.y << ", " << item.z;
    return output;
  }
};

template <template <typename _KeyType, typename _ValueType, size_t _Order, typename _Device>
          class Tree,
          typename KeyType, typename ValueType, typename Device>
using PointTypes = std::pair<BTreeContainer<KeyType, ValueType, 15, Device, Tree>, Device>;

template <typename C> class ContainerStructureTest : public testing::Test {
public:
  using Container = typename C::first_type;
  using Device = typename C::second_type;
};

using TreeTypes = ::testing::Types<PointTypes<BPlusTree, Point, uint32_t, Devices::Host>,
                                   PointTypes<BPlusTree, Point, uint32_t, Devices::Cuda>,
                                   PointTypes<BPlusTree, uint32_t, Point, Devices::Host>,
                                   PointTypes<BPlusTree, uint32_t, Point, Devices::Cuda>,
                                   PointTypes<BLinkTree, Point, uint32_t, Devices::Host>,
                                   PointTypes<BLinkTree, Point, uint32_t, Devices::Cuda>,
                                   PointTypes<BLinkTree, uint32_t, Point, Devices::Host>,
                                   PointTypes<BLinkTree, uint32_t, Point, Devices::Cuda>>;
TYPED_TEST_SUITE(ContainerStructureTest, TreeTypes);

template <typename Device>
void populatePoints(TNL::Containers::Array<Point, Device> &keys,
                    TNL::Containers::Array<uint32_t, Device> &values) {

  int size = keys.getSize();
  auto keysView = keys.getView();
  auto valuesView = values.getView();

  TNL::Algorithms::ParallelFor<Device>::exec(0, size, [=] __cuda_callable__(int i) mutable {
    Point point;
    point.x = point.y = point.z = i;
    keysView[i] = point;
    valuesView[i] = i;
  });
}

template <typename Device>
void populatePoints(TNL::Containers::Array<uint32_t, Device> &keys,
                    TNL::Containers::Array<Point, Device> &values) {

  int size = keys.getSize();
  auto keysView = keys.getView();
  auto valuesView = values.getView();

  TNL::Algorithms::ParallelFor<Device>::exec(0, size, [=] __cuda_callable__(int i) mutable {
    Point point;
    point.x = point.y = point.z = i;
    keysView[i] = i;
    valuesView[i] = point;
  });
}

TYPED_TEST(ContainerStructureTest, InsertRemove) {
  using Device = typename TestFixture::Device;
  using Container = typename TestFixture::Container;

  using KeyType = typename Container::NodeType::KeyType;
  using ValueType = typename Container::NodeType::ValueType;

  Container points(100);
  points.init();

  int size = 30;
  TNL::Containers::Array<KeyType, Device> keys(size);
  TNL::Containers::Array<ValueType, Device> values(size);

  TNL::Containers::Array<bool, Device> mask(size);
  TNL::Containers::Array<ValueType, Device> resultValues(size);

  populatePoints<Device>(keys, values);

  points.insert(keys, values);
  points.find(keys, resultValues, mask);

  ASSERT_EQ(values, resultValues);
  ASSERT_FALSE(mask.containsValue(false));

  points.remove(keys);
  points.find(keys, resultValues, mask);
  ASSERT_FALSE(mask.containsValue(true));
}

int main(int argc, char **argv) {
  std::ios::sync_with_stdio(false);
  testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();
}